This file will contain the basic BFM for driving HDLC packets into the DUT
<'
package sbt;

import sbt_packet;


//This unit looks after driving packets into the DUT.  There is a data
//and a frame pulse signal as part of of the driving protocol.  This unit
//also handles the injection of delays between packet according to the 
//packet_delay field contained within the packet itself.
unit sbt_bfm_u {

   //The clocking event will be pushed in from the 
   //agent
   event clock_e;

   //the bfm id will be pushed in from the parent agent
   bfm_id: uint;
   keep soft bfm_id == 0;

   //define hdl signals as strings for easy constraining in an 
   //environment 
   //data hdl signal
   sig_data: string;
   
   //frame pulse hdl signal
   sig_fp: string;

   //this field holds the current packet being driven into the dut
   !current_packet: sbt_packet_s;

   //This method will take in a packet and create a list of bytes to 
   //drive onto the input DUT lines.  It will toggle the frame pulse 
   //high for one clock cycle on the first byte of the packet.  At the 
   //end of driving a packet, it will also tristate the data lines while
   //waiting for a fixed delay before sending packets.  The value for the 
   //number of clocks to wait in the delay is extracted from the packet
   //itself.  There are also a number of "hook methods" for the user to 
   //extend to add functionality.
   drive(packet_to_drive: sbt_packet_s)@clock_e is {
      
      //store the packet in a temporary location 
      //for the environment to use
      current_packet = packet_to_drive;

      //pack the higher level data structure into a list of bits 
      //for a complete explanation of the pack method, refer to the 
      //online specman docs
      var packed_packet: list of byte = pack(packing.low, packet_to_drive);
      
      //set the frame pulse signal high
      '(sig_fp)' = 1'b1;      
      
      //drive each bit out onto the data signal one bit at a time, on every 
      //clock edge
      for each in packed_packet do {
         
         //hook method
         just_before_driving();
         
         //drive the contents of the packet onto the line one
         //item at a time.
         '(sig_data)' = it;
         
         //hook method
         just_after_driving();
         
         wait cycle;
      }; // for each in pac...
      
      //set the frame pulse signal low
      '(sig_fp)' = 1'b0;
      
      //drive high impedance onto the lines
      '(sig_data)@z' = 8'hFF;

      //wait for a delay before next packet is accepted
      wait [packet_to_drive.packet_delay]*cycle;

   }; // drive(packet_to...
   
   //hook methods for user extension
   just_before_driving()@clock_e is empty;
   just_after_driving()@clock_e is empty;


}; // unit sbt_bf...

'>
